//*************************************************************************
// HTML Validator
//
//  File: tidyBrowser.js
//  Description: javascript to validate the HTML in the browser window
//  Author : Marc Gueury
//  Licence : see licence.txt
//*************************************************************************

//-------------------------------------------------------------

const UC_CTRID = "@mozilla.org/intl/scriptableunicodeconverter";
const nsISupports = Components.interfaces.nsISupports;
const nsIIOService = Components.interfaces.nsIIOService;
const nsIUnicodeConverter = Components.interfaces.nsIScriptableUnicodeConverter;

//-------------------------------------------------------------

var oTidyBrowser = null;
var oTidyTabChangeObserver = null;
onLoadTidyBrowser();

//-------------------------------------------------------------

function onLoadTidyBrowser()
{
  onLoadTidyUtil();
  oTidyBrowser = new TidyBrowser();
  window.top.removeEventListener("load", onTidyBrowserTopLoad, true);
  window.top.addEventListener("load", onTidyBrowserTopLoad, true);
  window.top.removeEventListener("pageshow", onTidyBrowserTopPageShow, true);
  window.top.addEventListener("pageshow", onTidyBrowserTopPageShow, true);
}

function onUnloadTidyBrowser()
{
  delete oTidyBrowser;
  oTidyBrowser = null;

  var main_frame = document.getElementById("content");
  main_frame.removeProgressListener(oTidyTabChangeObserver);
}

function onTidyBrowserClicked(event) 
{
  return;
}

function onTidyBrowserCopyHtmlToClipboard() 
{
  try
  {
    const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
    var html = oTidyBrowser.getHtmlFromCache( window.content.document, false );
    clipboardHelper.copyString(html);
  } 
  catch(e) 
  {
    // do nothing, later code will handle the error
    dump("Unable to get the clipboard helper\n");
  }
}

function onTidyBrowserCopyErrorToClipboard() 
{
  try
  {
    if( !oTidyBrowser.isLoading() )
    {
      var data = oTidyUtil.getString("tidy_result")+"\n----------------------\n"+oTidyBrowser.getErrorList(window.content);
      const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
      clipboardHelper.copyString(data);
    }
  } 
  catch(e) 
  {
    // do nothing, later code will handle the error
    dump("Unable to get the clipboard helper\n");
  }
}

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

function TestLinks() 
{
  const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
  var main_frame=window.content;
  var data = main_frame.document.URL+"\n\n";
  var html = oTidyBrowser.getHtmlFromCache( main_frame.document, false );

  // The inout arguments need to be JavaScript objects
  var nbError = {value:0};
  var nbWarning = {value:0};
  var nbAccessWarning = {value:0};
  var nbHidden = {value:0};
  var links ={value:"---"};
  var accessLevel = oTidyUtil.getIntPref( "accessibility-check" );

  oTidyUtil.tidy.getLinks( html, oTidyUtil.getPrefConfig(), accessLevel, links, nbError, nbWarning, nbAccessWarning, nbHidden );
  data += links.value + "\n";

  var res = new TidyResult();
  res.iNbError = nbError.value;
  res.iNbWarning = nbWarning.value;
  res.iNbAccessWarning = nbAccessWarning.value;
  res.iNbHidden = nbHidden.value;

  clipboardHelper.copyString(data);
}

//-------------------------------------------------------------
// TidyTabChangeObserver
//-------------------------------------------------------------

function TidyTabChangeObserver()
{
}

TidyTabChangeObserver.prototype =
{
  onProgressChange : function (aWebProgress, aRequest,
                                  aCurSelfProgress, aMaxSelfProgress,
                                  aCurTotalProgress, aMaxTotalProgress) 
  {},

  onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus)
  {},

  onLocationChange : function(aWebProgress, aRequest, aLocation) 
  {
    // XXXXXXXXXX
    if( aWebProgress.isLoadingDocument==false )
    {
      // alert( 'test' );
      oTidyBrowser.validateFrame( window.content );
      oTidyBrowser.updateStatusBar();
    }
  },

  onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage) 
  {},

  onSecurityChange : function(aWebProgress, aRequest, aState) 
  {},
  
  onLinkIconAvailable : function(aBrowser, aHref) 
  {},

  QueryInterface : function(aIID)
  {
    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsISupports))
    return this;
    throw Components.results.NS_NOINTERFACE;
  }
}

//-------------------------------------------------------------
// TidyBrowser
//-------------------------------------------------------------

function TidyBrowser()
{
  try
  {
    this.initIcon();
  }
  catch(ex)
  {
  }
}

TidyBrowser.prototype =
{  
  xulBrowserHbox  : null,
  xulBrowserImg   : null,
  xulBrowserLbl   : null,
  xulMenuIconText : null,
  xulMenuIconOnly : null,
  xulMenuDisable  : null,
  xulMenuHtmlClipboard : null,
  iCounter : 0,
  bIgnorePageShow : false,

  /**
   * initIcon
   */
  initIcon : function()
  {
    var browser_icon = oTidyUtil.getCharPref( "browser_icon" );
    var browser_disable = oTidyUtil.getBoolPref( "browser_disable" );

    // checkbox
    this.xulMenuIconText = document.getElementById('tidy.browser.menu.icon_text');
    this.xulMenuIconOnly = document.getElementById('tidy.browser.menu.icon_only');
    this.xulMenuDisable = document.getElementById('tidy.browser.menu.disable');
    this.xulMenuHtmlClipboard = document.getElementById('tidy.browser.menu.html_clipboard');

    // Icon and text    
    this.xulBrowserHbox = document.getElementById("tidy-status-bar-hbox");
    this.xulBrowserImg  = document.getElementById("tidy-status-bar-img");
    this.xulBrowserLbl  = document.getElementById("tidy-browser-error");
    
    this.validateFrame( window.content );
    this.updateStatusBar();
  },
    
  /** __ updateStatusBar __________________________________________________________
   */ 
  updateStatusBar : function()
  {    
    var tooltip, icon, parent, lblparent, lbl;

    var browser_disable = oTidyUtil.getBoolPref( "browser_disable" );
    var browser_icon = oTidyUtil.getCharPref( "browser_icon" );

    if (this.xulBrowserHbox.hasAttribute("tooltip")) 
    {
      // In Firefox 1.1, avoid several tooltips
      tooltip = document.getElementById("tidy.browser.hbox.tooltip");
      while( tooltip!=null )
      {
        tooltip.parentNode.removeChild(tooltip);
        this.xulBrowserHbox.removeAttribute("tooltip");
        tooltip = document.getElementById("tidy.browser.hbox.tooltip");
      }
    }      

    this.xulMenuIconText.setAttribute("checked", browser_icon=="icon_text");
    this.xulMenuIconOnly.setAttribute("checked", browser_icon!="icon_text");
    this.xulMenuDisable.setAttribute("label", oTidyUtil.getString( (browser_disable?"tidy_enable":"tidy_disable") ));

    // Maybe to uncomment in a next version
    this.xulMenuHtmlClipboard.hidden = !oTidyUtil.getBoolPref( "debug" );

    this.xulBrowserImg.hidden = false;
    this.xulBrowserLbl.hidden = false;
        
    if( browser_icon!="icon_text" )
    {
      this.xulBrowserLbl.hidden = true;
    }
    
    var img = this.xulBrowserImg;
    if( browser_disable )
    {
      icon = "chrome://tidy/skin/disabled";
      this.xulBrowserLbl.hidden = true;
      
      if( browser_icon=="hide" )
      {
        var panel = document.getElementById('tidy-statusbar-panel');
        panel.hidden = true;
      }      
    }
    else
    {
      ////
      //// Sum
      ////
      var pageResult = window.content.document.tidyResult;
      var iNbFrame = this.getNbFrame( window.content );

      var sum = new TidyResult( null ); // virtual result
      sum.bInDomainList = false; 
      sum.bEmpty = true; 
      this.sumResult( sum, window.content );

      var str = sum.getErrorString();
      if( oTidyUtil.getBoolPref( "debug" ) )
      {
        str += " #"+this.iCounter + "/"+iNbFrame;
      }
      this.xulBrowserLbl.value = str;      

      ////
      //// Icon in status bar
      ////
      icon = sum.getErrorIcon();    
    }
    img.setAttribute("src", icon + ".png" );
    
    ////
    //// tooltip ( create empty tip / populate it )
    ////       
    parent = document.getElementById("tidy-dummy-popup");
    tooltip = document.createElement("tooltip");
    tooltip.setAttribute("id", "tidy.browser.hbox.tooltip");
    tooltip.setAttribute("noautohide", true);

    var hbox_main = document.createElement("hbox");
    var vbox1 = document.createElement("vbox");
    var vbox2 = document.createElement("vbox");
    hbox_main.appendChild(vbox1);
    hbox_main.appendChild(vbox2);
       
    var img1 = document.createElement("image");
    img1.setAttribute("src", "chrome://tidy/skin/tidy_logo_big.png");
    img1.setAttribute("width", "59" );
    img1.setAttribute("height","55" );
    vbox1.appendChild(img1);

    lbl = document.createElement("label");

    if( browser_disable || sum.bEmpty )
    {
      lbl.setAttribute("value", oTidyUtil.getString("tidy_validator") );
      vbox2.appendChild(lbl);
      lbl = document.createElement("label");
      lbl.setAttribute("value", (browser_disable?oTidyUtil.getString("tidy_disabled"):oTidyUtil.getString('tidy_empty')) );
      vbox2.appendChild(lbl);
    }
    else
    {
      lbl.setAttribute("value", oTidyUtil.getString("tidy_result") );
      vbox2.appendChild(lbl);

      if( !sum.bInDomainList )
      {
        this.xulBrowserLbl.hidden = true;
      }

      if( iNbFrame==0 )
      {
        if( sum.bInDomainList )
        {
          var hbox1 = document.createElement("hbox");
          vbox2.appendChild(hbox1);
          var vbox3 = document.createElement("vbox");
          hbox1.appendChild(vbox3);

          var img2 = document.createElement("image");
          img2.setAttribute("src", icon + "_big.png");
          img2.setAttribute("width", "32" );
          img2.setAttribute("height","32" );
          vbox3.appendChild(img2);

          var vbox4 = document.createElement("vbox");
          hbox1.appendChild(vbox4);

          lbl = document.createElement("label");
          lbl.setAttribute("value", sum.iNbError+" "+oTidyUtil.getString( sum.getPluralString(sum.iNbError,"error") ));
          vbox4.appendChild(lbl);

          lbl = document.createElement("label");
          lbl.setAttribute("value", sum.iNbWarning+" "+oTidyUtil.getString( sum.getPluralString(sum.iNbWarning,"warning") ));
          vbox4.appendChild(lbl);

          if( sum.iNbAccessWarning!=0 )
          {
            lbl = document.createElement("label");
            lbl.setAttribute("value", sum.iNbAccessWarning+" "+oTidyUtil.getString( sum.getPluralString(sum.iNbAccessWarning,"access_warning") ));
            vbox4.appendChild(lbl);    
          }
          if( sum.iNbHidden!=0 )
          {
            lbl = document.createElement("label");
            lbl.setAttribute("value", sum.iNbHidden+" "+oTidyUtil.getString("tidy_hidden"));
            vbox4.appendChild(lbl);    
          }
        }
        else
        {
          lbl = document.createElement("label");
          lbl.setAttribute("value", oTidyUtil.getString("tidy_not_in_domain")+":");
          vbox2.appendChild(lbl);          
          lbl = document.createElement("label");
          lbl.setAttribute("value", oTidyUtil.getCharPref("validate_domain_list") );
          vbox2.appendChild(lbl);          
        }
      }
      else 
      {
        this.addIconLabelToVbox( vbox2, window.content );
      }
    } 
    
    tooltip.setAttribute("orient", "horizontal");
    tooltip.appendChild(hbox_main);

    // append new tooltip
    parent.appendChild(tooltip);
    this.xulBrowserHbox.setAttribute("tooltip", "tidy.browser.hbox.tooltip");    
  },
  /** __ isLoading _________________________________________________________________
   *
   * return true if the current browser is still loading
   */
  isLoading : function()
  {   
    var res = document.getElementById("content").mCurrentBrowser.webProgress.isLoadingDocument;
    if( res )
    {
      alert( oTidyUtil.getString("tidy_browser_loading") );
    }
    return res;
  },
  
  /** __ getHtmlFromCache __________________________________________________________
   * 
   * get the HTML of the current window
   */ 
  getHtmlFromCache : function( doc, safe_call )
  {   
    // If the call is not safe, check that the page is not loading to prevent hanging
    if( !safe_call )
    {
      if( this.isLoading() )
      {
        return '';
      }
    }
  
    oTidyUtil.tidy.log( '<JAVASCRIPT>getHtmlFromCache : ' + doc.URL );
    
    // Cleanup with a lost tidyResult : icon (?)
    if( doc.tidyResult==null )
    {
      doc.tidyResult = new TidyResult( doc );
    }
    doc.tidyResult.bEmpty = true;

    ////
    //// Part 1 : get the history entry (nsISHEntry) associated with the document
    ////
    var webnav;
    try 
    {
        // Get the DOMWindow for the requested document.  If the DOMWindow
        // cannot be found, then just use the _content window...
        //
        // XXX:  This is a bit of a hack...
        var win = doc.defaultView;
        if (win == window) 
        {
          win = _content;
        }
        var ifRequestor = win.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
        webNav = ifRequestor.getInterface(Components.interfaces.nsIWebNavigation);
    } catch(err) {
        // If nsIWebNavigation cannot be found, just get the one for the whole
        // window...
        webNav = getWebNavigation();
    }
    //
    // Get the 'PageDescriptor' for the current document. This allows the
    // to access the cached copy of the content rather than refetching it from 
    // the network...
    try
    {
      var PageLoader = webNav.QueryInterface(Components.interfaces.nsIWebPageDescriptor);
      var pageCookie = PageLoader.currentDescriptor;     
      var shEntry = pageCookie.QueryInterface(Components.interfaces.nsISHEntry);
    } catch(err) {
      // If no page descriptor is available, just use the URL...
    }  

    ////
    //// Part 2 : open a nsIChannel to get the HTML of the doc
    ////
    var url = doc.URL;
    var urlCharset = doc.characterSet;
    
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
    var channel = ios.newChannel( url, urlCharset, null );
    channel.loadFlags |= Components.interfaces.nsIRequest.VALIDATE_NEVER;
    channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_FROM_CACHE;
    channel.loadFlags |= Components.interfaces.nsICachingChannel.LOAD_ONLY_FROM_CACHE;
        
    // Bad trick : Avoid problem with file directory
    try
    {
      if( channel.contentType=="application/x-unknown-content-type" )
      {
        return("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"><html><head><title></title></head><body></body></html>");
      }
    } 
    catch(e) 
    {
    }

    try
    {
      // Use the cache key to distinguish POST entries in the cache (see nsDocShell.cpp)
      var cacheChannel = channel.QueryInterface(Components.interfaces.nsICachingChannel);
      cacheChannel.cacheKey = shEntry.cacheKey;
    } 
    catch(e) 
    {
    }

    var stream = channel.open();

    const scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
    scriptableStream.init( stream );
    var s = "";
    while( scriptableStream.available()>0 )
    {
      doc.tidyResult.bEmpty = false;
      s += scriptableStream.read(scriptableStream.available());
    }
    scriptableStream.close();    
    stream.close();

    ////
    //// Part 3 : convert the HTML in unicode
    ////  
    try
    {
      var ucConverter =  Components.classes[UC_CTRID].getService(nsIUnicodeConverter);
      ucConverter.charset = urlCharset;
      var s2 = ucConverter.ConvertToUnicode( s );
    }
    catch(e) 
    {
      doc.tidyResult.bUConvFailed = true;
    }

    oTidyUtil.tidy.log( '</JAVASCRIPT>getHtmlFromCache' );
    return( s2 );
  },
  
  /** __ validateCacheHtml ______________________________________________________
   * 
   * Validate the HTML and add the results in the tree
   */    
  validateCacheHtml : function( aHtml, aResult )
  {
    // Set the initialization flag
    this.iCounter = this.iCounter + 1;
    
    // Load the translation (if not done)
    oTidyUtil.translation();

    // The inout arguments need to be JavaScript objects
    var nbError = {value:0};
    var nbWarning = {value:0};
    var nbAccessWarning = {value:0};
    var nbHidden = {value:0};
    var error ={value:"---"};
    var accessLevel = oTidyUtil.getIntPref( "accessibility-check" );
    
    oTidyUtil.tidy.getErrorsInHTML( aHtml, oTidyUtil.getPrefConfig(), accessLevel, error, nbError, nbWarning, nbAccessWarning, nbHidden );
    // alert( "error: " + nbError.value + " / warning : " + nbWarning.value );

    aResult.iNbError = nbError.value;
    aResult.iNbWarning = nbWarning.value;
    aResult.iNbAccessWarning = nbAccessWarning.value;
    aResult.iNbHidden = nbHidden.value;
    aResult.bValidated = true;
  },
  
  /** __ getResult ______________________________________________________
   */    
  getResult : function( doc )
  {
    var bFrame = false;
    // check that it is the current document seen
    if( doc!=window.content.document ) 
    {
      /// XXXXXXXXXXXXXXX To verify XXXXXXXXXXX 
      if( doc.tidyResult==null && this.isDocumentFrameOf( doc, window.content ) ) 
      {
        bFrame = true;
      }
      else
      {
        return null;
      }
    }    
    
    // check that the main page is changed
    // a frame can be validated before the main page
    var pageDoc = window.content.document;
    var pageResult = new TidyResult( pageDoc );
    // var bChanged = pageDoc.tidyResult==null || !pageDoc.tidyResult.compare( pageResult );
    var bChanged = pageDoc.tidyResult==null;
    if( bChanged )
    {
      pageResult.bInDomainList = oTidyUtil.isInDomainList( pageDoc );    
      pageDoc.tidyResult = pageResult;
    }

    if( bFrame )
    {
      var frameResult = new TidyResult( doc );
      doc.tidyResult = frameResult;
      return frameResult;
    }
    else
    {
      if( !pageDoc.tidyResult.bValidated )
      {
        return pageDoc.tidyResult;
      }
      return null;
    }
  }, 
    
  /** __ validateCache ______________________________________________________
   */    
  validateCache : function( doc )
  {
    if( oTidyUtil.getBoolPref("browser_disable") )
    {
      return;
    }
    
    // doc = window.content.document for the main frame
    if( doc.contentType == "text/html"
     || doc.contentType == "application/xhtml+xml"
    ) 
    {
      var result = this.getResult( doc );
      if( result!=null )
      {
        result.bInDomainList = oTidyUtil.isInDomainList( doc );    
        if( result.bInDomainList )
        {
          var html = this.getHtmlFromCache( doc, true );
          if( !result.bEmpty )
          {
            this.validateCacheHtml( html, result );
          }
        }
      } 
    }
    this.updateStatusBar();
  },

  /** __ getNbFrame ______________________________________________________
   */    
  getNbFrame : function( main_frame )
  {
    const framesList = main_frame.frames;
    var r = framesList.length;
    for(var i = 0; i < framesList.length; i++)
    {
      r += this.getNbFrame( framesList[i] );
    }
    return r;
  },

  /** __ validateFrame ______________________________________________________
   */    
  validateFrame : function( main_frame )
  {
    if( main_frame.document!=null && main_frame.document.tidyResult==null )
    {
      this.validateCache( main_frame.document );
    }
    // Loop through the frames
    var framesList = main_frame.frames;
    for(var i = 0; i < framesList.length; i++)
    {
      this.validateFrame( framesList[i] );
    }
  },
  
  /** __ addIconLabelToVbox ______________________________________________________
   */    
  addIconLabelToVbox : function( vbox2, main_frame )
  {
    var hbox1 =  document.createElement("hbox");
    
    var icon = "chrome://tidy/skin/question";
    var str = "";

    var res = main_frame.document.tidyResult;
    if( res!=null )
    {
      icon = res.getErrorIcon();
      str = res.getErrorString();
    }
    str = oTidyUtil.getString((main_frame==window.content?"tidy_page":"tidy_frame"))+ ": "+str;

    var img2 = document.createElement("image");
    img2.setAttribute("src", icon + ".png");
    img2.setAttribute("width", "16" );
    img2.setAttribute("height","16" );
    hbox1.appendChild(img2);

    lbl = document.createElement("label");
    lbl.setAttribute("value", str );
    hbox1.appendChild(lbl);

    vbox2.appendChild(hbox1);
    
    // Loop through the frames
    const framesList = main_frame.frames;
    for(var i = 0; i < framesList.length; i++)
    {
      this.addIconLabelToVbox( vbox2, framesList[i] );
    }
  },


  /** __ sumResult ______________________________________________________
   */    
  sumResult : function( sum, main_frame )
  {
    var res = main_frame.document.tidyResult;
    if( res!=null )
    {
      sum.iNbError += res.iNbError;
      sum.iNbWarning += res.iNbWarning;
      sum.iNbAccessWarning += res.iNbAccessWarning;
      sum.iNbHidden += res.iNbHidden;
      sum.bInDomainList |= res.bInDomainList;
      sum.bUConvFailed |= res.bUConvFailed;
      sum.bEmpty &= res.bEmpty;
    }

    // Loop through the frames
    var framesList = main_frame.frames;
    for(var i = 0; i < framesList.length; i++)
    {
      this.sumResult( sum, framesList[i] );
    }
  },
  
  /** __ isDocumentFrameOf ______________________________________________________
   */    
  isDocumentFrameOf : function( doc, main_frame )
  {
    const framesList = main_frame.frames;

    // Loop through the frames
    for(var i = 0; i < framesList.length; i++)
    {
      if( framesList[i].document == doc ) 
      {
        return true;
      }
      if( this.isDocumentFrameOf( doc, framesList[i] ) )
      {
        return true;
      }
    }

    return false;
  },
  
  /** __ getErrorList ______________________________________________________
   */    
  getErrorList : function( main_frame )
  {
    var data = main_frame.document.URL+"\n\n";
    var html = this.getHtmlFromCache( main_frame.document, false );

    // The inout arguments need to be JavaScript objects
    var nbError = {value:0};
    var nbWarning = {value:0};
    var nbAccessWarning = {value:0};
    var nbHidden = {value:0};
    var error ={value:"---"};
    var accessLevel = oTidyUtil.getIntPref( "accessibility-check" );

    oTidyUtil.tidy.getErrorsInHTML( html, oTidyUtil.getPrefConfig(), accessLevel, error, nbError, nbWarning, nbAccessWarning, nbHidden );
    data += error.value + "\n";

    var res = new TidyResult();
    res.iNbError = nbError.value;
    res.iNbWarning = nbWarning.value;
    res.iNbAccessWarning = nbAccessWarning.value;
    res.iNbHidden = nbHidden.value;
    if( res.iNbError>=oTidyUtil.getIntPref("show-errors") )
    {
      data += oTidyUtil.getString("tidy_too_many_error");
    }
    data += res.getErrorString()+"\n\n";

    // Loop through the frames
    var framesList = main_frame.frames;
    for(var i = 0; i < framesList.length; i++)
    {
      data += this.getErrorList( framesList[i] );
    }
    return data;
  }, 
  
  /** __ findDocWithURL ______________________________________________________
   */    
  findDocWithURL : function( url, main_frame )
  {
    if( main_frame.document.URL==url ) 
    {
      return main_frame.document;
    }

    // Loop through the frames
    const framesList = main_frame.frames;
    for(var i = 0; i < framesList.length; i++)
    {
      var res = this.findDocWithURL( url, framesList[i] );
      if( res!=null ) 
      {
        return res;
      }
    }
    return null;
  },  
  
  /** __ onViewSource ______________________________________________________
   */    
  onViewSource : function(aEvent)
  {
    if( oTidyUtil.getBoolPref( "browser_disable" ) ) 
    {
      // If the validation is disabled, ask to reenable it
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                          .getService(Components.interfaces.nsIPromptService);
      var flags=promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0 +
                promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1;
      var res = promptService.confirmEx(window,oTidyUtil.getString("tidy_validator"),
                 oTidyUtil.getString("tidy_enable_browser"),
                 flags, null, null, null, null, {} );

      if( res==0 )
      {   
        oTidyUtil.setBoolPref( "browser_disable", false );
        this.initIcon();
      }
    }
    else
    {
      if( this.getNbFrame(window.content)==0 )
      {
        this.viewSourceOfDocument(window.content.document, aEvent);
      }
      else
      {
        var pageResult = window.content.document.tidyResult;
        var result = new Array(); 
        var aType = (aEvent.originalTarget.id=="tidy.browser.menu.cleanup")?'cleanup':'viewsource';
        openDialog(
                   "chrome://tidy/content/tidyChooseSource.xul",
                   "",
                   "centerscreen,dialog=no,chrome,resizable,dependent,modal",
                   window.content,
                   result,
                   aType
                  );
        for( var i=0; i<result.length; i++ )
        {
          var doc = this.findDocWithURL( result[i], window.content );
          if( doc )
          {
            this.viewSourceOfDocument(doc, aEvent);
          }
        }        
      }
    }
  },
  
  
  /** __ viewSourceOfDocument ______________________________________________________
   *
   * !! Nearly a copy paste of BrowserViewSourceOfDocument
   */    
  viewSourceOfDocument: function(aDocument, aEvent)
  {
    var pageCookie;
    var webNav;
  
    // Get the document charset
    var docCharset = "charset=" + aDocument.characterSet;
  
    // Get the nsIWebNavigation associated with the document
    try 
    {
        // Get the DOMWindow for the requested document.  If the DOMWindow
        // cannot be found, then just use the _content window...
        //
        // XXX:  This is a bit of a hack...
        var win = aDocument.defaultView;
        if (win == window) 
        {
          win = _content;
        }
        var ifRequestor = win.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
        webNav = ifRequestor.getInterface(nsIWebNavigation);
    } catch(err) {
        // If nsIWebNavigation cannot be found, just get the one for the whole
        // window...
        webNav = getWebNavigation();
    }
    //
    // Get the 'PageDescriptor' for the current document. This allows the
    // view-source to access the cached copy of the content rather than
    // refetching it from the network...
    //
    try
    {
      var PageLoader = webNav.QueryInterface(Components.interfaces.nsIWebPageDescriptor);
  
      pageCookie = PageLoader.currentDescriptor;
    } catch(err) {
      // If no page descriptor is available, just use the view-source URL...
    }

    var id = aEvent.originalTarget.id;
    if( id == "tidy.browser.menu.cleanup" ) 
    {
      var win_arg = new Array();
      win_arg[0] = webNav.currentURI.spec;
      win_arg[1] = docCharset;
      win_arg[2] = pageCookie;
      var sHtml = this.getHtmlFromCache(aDocument, false);

      if (aDocument.tidyResult == null)
      {
        this.validateCache(aDocument);    
      }
      oTidyUtil.cleanupDialog(aDocument.tidyResult.iNbError, sHtml, win_arg);
    }
    else 
    {
      BrowserViewSourceOfURL(webNav.currentURI.spec, docCharset, pageCookie, aEvent);
    }

    /*
    var win_arg = new Array();
    win_arg[0] = webNav.currentURI.spec;
    win_arg[1] = docCharset;
    win_arg[2] = pageCookie;
    var sHtml = this.getHtmlFromCache( aDocument, false );
    if( aDocument.tidyResult==null )
    {
      this.validateCache( aDocument );    
    }
    
    oTidyUtil.cleanupDialog( aDocument.tidyResult.iNbError, sHtml, win_arg );
    */
  },
  
  /** __ onAbout ______________________________________________________
   */    
  onAbout : function()
  {
    openDialog(
               "chrome://tidy/content/tidyAbout.xul",
               "",
               "centerscreen,dialog,chrome,dependent"
              );
  },

  /** __ onOptions ______________________________________________________
   */    
  onOptions : function()
  {
    openDialog(
               "chrome://tidy/content/tidyOptions.xul",
               "",
               "centerscreen,dialog=no,chrome,resizable,dependent,modal"
              );
    this.initIcon();
  },

  /** __ onIconText ______________________________________________________
   */    
  onIconText : function()
  {
    oTidyUtil.setCharPref( "browser_icon", "icon_text" );
    this.initIcon();
  },


  /** __ onIconOnly ______________________________________________________
   */    
  onIconOnly : function()
  {
    oTidyUtil.setCharPref( "browser_icon", "icon" );
    this.initIcon();
  },


  /** __ onDisable ______________________________________________________
   */    
  onDisable : function()
  {
    oTidyUtil.setBoolPref( "browser_disable", !oTidyUtil.getBoolPref( "browser_disable" ) );
    this.initIcon();
  }
}

//----------------------------------------------------------------------------------------------

function onTidyBrowserTopLoad( event ) 
{
  try
  {  
    try
    {
      oTidyBrowser.bIgnorePageShow = true;
      oTidyBrowser.validateCache(  event.originalTarget );
    }
    catch(ex)
    {
      delete oTidyBrowser;
      if( oTidyBrowser==null )
      {
        oTidyBrowser = new TidyBrowser();
      }
      else
      {
        oTidyBrowser = null;
      }
      oTidyBrowser.validateCache(  event.originalTarget );

      if( oTidyTabChangeObserver==null )
      {
        oTidyTabChangeObserver = new TidyTabChangeObserver();
        var main_frame = document.getElementById("content");
        main_frame.addProgressListener(oTidyTabChangeObserver, Components.interfaces.nsIWebProgress.NOTIFY_ALL);    
      }
    }
  }
  catch(ex2)
  {}
}

/** 
 * New event in Firefox 1.4 
 * If the extension was Firefox 1.4 only onTidyBrowserTopLoad could be removed
 */
function onTidyBrowserTopPageShow( event ) 
{
  try
  { 
    if( !oTidyBrowser.bIgnorePageShow )
    {
      oTidyBrowser.updateStatusBar();
      // Work-around for bug 312027
      // onTidyBrowserTopLoad( event );
    }
    oTidyBrowser.bIgnorePageShow = false;
  }
  catch(ex)
  {}  
}
