//*************************************************************************
// HTML Validator
//
//  File: tidyOptions.js
//  Description: javascript for the dialog box of the options window
//  Author : Marc Gueury
//  Licence : see licence.txt
//*************************************************************************

//-------------------------------------------------------------

var oTidyOptions;

function onLoadTidyOptions()
{
  onLoadTidyUtil();
  oTidyOptions = new TidyOptions();
  oTidyOptions.start();  
}

function onUnloadTidyOptions()
{
  onUnloadTidyUtil();
  delete oTidyOptions;
  oTidyOptions = null;
}

//-------------------------------------------------------------
// oTidyOptions
//-------------------------------------------------------------

function TidyOptions()
{
}

TidyOptions.prototype =
{
  // Accessibility check combobox
  accessList: null, 
  browserIconList: null,
  filterHideList: null,
  validateDomainTypeList: null,
  validateDomainList: null,

  // Initialisation and termination functions
  start : function()
  {
    this.accessList = document.getElementById("tidy.options.accessibility-check");
    this.browserIconList = document.getElementById("tidy.options.browser_icon");
    this.filterHideList = document.getElementById("tidy.options.filter.list.hide");
    this.validateDomainTypeList = document.getElementById("tidy.options.validate_domain_type");
    this.validateDomainList = document.getElementById("tidy.options.validate_domain_list");
    
    oTidyUtil.initCheckbox( "browser_disable" );
    
    var type = oTidyUtil.getCharPref("validate_domain_type");
    this.validateDomainTypeList.selectedIndex = (type=="all"?0:(type=="only"?1:2));
    this.validateDomainList.disabled = (type=="all");
    
    this.browserIconList.selectedIndex = (oTidyUtil.getCharPref("browser_icon")=="icon_text"?0:1);
    
    oTidyUtil.initCheckbox( "viewsource_disable" );
    oTidyUtil.initCheckbox( "highlight_line" );
    oTidyUtil.initCheckbox( "show-warnings" );   
    oTidyUtil.initCharTextbox ( "validate_domain_list" );

    this.accessList.selectedIndex = oTidyUtil.getIntPref( "accessibility-check" ) + 1;

    // Filter List
    for( var o in oTidyUtil.filterArray )
    {
      if( oTidyUtil.filterArray[o]==false )
      {
        // The inout arguments need to be JavaScript objects
        var aErrorDesc = {value:""};
        oTidyUtil.tidy.getErrorDescription( o, aErrorDesc );

        this.filterHideList.appendItem( aErrorDesc.value, o );
      }
    }
    
  },

  onFilterRemove : function()
  {
    var index = this.filterHideList.selectedIndex;
    if( index>=0 )
    {
      this.filterHideList.removeItemAt ( index );
    }  
  },

  /**
   * Enable or disable the domain list
   */
  domainList : function( enabled )
  {
    this.validateDomainList.disabled = !enabled;
  },

  onOk : function()
  {
    oTidyUtil.saveCheckbox( "browser_disable" );
    oTidyUtil.saveCheckbox( "viewsource_disable" );
    oTidyUtil.saveCheckbox( "highlight_line" );
    oTidyUtil.saveCheckbox( "show-warnings" );
    oTidyUtil.saveCharTextbox ( "validate_domain_list" ); 

    // Domain type : Convert the combobox to value
    var index = this.validateDomainTypeList.selectedIndex;
    oTidyUtil.setCharPref( "validate_domain_type", (index==0?"all":(index==1?"only":"except")) );
    
    // Icon: Convert the combobox to value
    oTidyUtil.setCharPref( "browser_icon", (this.browserIconList.selectedIndex==0?"icon_text":"icon_only") );

    // Force output : convert the combobox to 2 settings of tidy
    oTidyUtil.setIntPref( "accessibility-check", this.accessList.selectedIndex-1 );
        
    // Save the hide list in the filterArray
    oTidyUtil.filterArray = new Array();
     
    if( this.filterHideList.getRowCount()>0 )
    {
      var item = this.filterHideList.getItemAtIndex( 0 );
      while( item )
      {
        oTidyUtil.filterArray[item.value] = false;
        item = this.filterHideList.getNextItem ( item, 1 )
      }   
    }
    oTidyUtil.saveFilterArrayInPref();

    // Close the window
    window.close();
  }
}