//*************************************************************************
// HTML Validator
//
//  File: tidyThread.js
//  Description: javascript to manage threads
//  Author : Marc Gueury
//  Licence : see licence.txt
//*************************************************************************

function TestThread() 
{
  // var validateRunner = new SyncRunner();
  var validateRunner = new AsyncRunner();
  // var validateRunner = new AsyncRunner(new LastRunsStarter());
  const mainJob=function() 
  {
    oTidyUtil.tidy.log( 'mainjob' );
  }
  const postJob=function() 
  { 
    oTidyUtil.tidy.log( 'postjob' );
  }
  validateRunner.run(mainJob,postJob);
  oTidyUtil.tidy.log( 'mainthread' );
}


//- Factory DEF_THREAD_FACTORY ------------------------------------------------
const DEF_THREAD_FACTORY = 
{
  createThread: function(runnable) 
  {
    const cc=Components.classes;
    const ci=Components.interfaces;
    const thread=cc['@mozilla.org/thread;1'].createInstance(ci.nsIThread);
    if(runnable) 
    {
      runnable.thread=thread;
      thread.init(runnable, 0,  
            ci.nsIThread.PRIORITY_NORMAL, ci.nsIThread.SCOPE_GLOBAL, ci.nsIThread.STATE_JOINABLE);
    }
    return thread;
  },

  get currentThread() 
  {
    return DEF_THREAD_FACTORY.createThread().currentThread;
  }
}

//- Class SimpleStarter ------------------------------------------------
function SimpleStarter(threadFactory) 
{
  this.threadFactory=threadFactory?threadFactory:DEF_THREAD_FACTORY;
}
SimpleStarter.prototype = 
{
  start: function(runnable) 
  {
    this.threadFactory.createThread(runnable);
  }
}

//- Class LastRunsStarter ------------------------------------------------
function LastRunsStarter(threadFactory) 
{
  this.threadFactory=threadFactory?threadFactory:DEF_THREAD_FACTORY;
  this.threadFactory.createThread(this);
}
LastRunsStarter.prototype = 
{
  current: null,
  running: true,
  run: function() 
  {
    while(this.running) 
    {
      if(this.current && this.current.run) 
      {
        this.current.run();
        this.current=null;
      }
      this.thread.sleep(100);
    }
    delete this.thread;
  },
  
  start: function(runnable) 
  {
    this.current=runnable;
  }
}

//- Class SyncRunner ------------------------------------------------
function SyncRunner() {}
SyncRunner.prototype=
{
  run: function(mainJob,postJob) 
  {
    try 
    {
      mainJob();
    } catch(ex) {
       dump("SyncRunner mainJob error: "+ex+"\n");
    }
    try 
    {
      if(postJob) postJob();
    } catch(ex) {
      dump("SyncRunner postJob error: "+ex+"\n");
    }
  }
}

//- Class AsyncRunner ------------------------------------------------
function AsyncRunner(starter) 
{
  if(starter) this.starter=starter;
}
AsyncRunner.prototype=
{
  starter: new SimpleStarter(),
 
  run: function(asyncJob,postJob) 
  {
    const cc=Components.classes;
    const ci=Components.interfaces;
  
    var postRunnable=null;
    const asyncError={ value: null };
    if(postJob) 
    {
      postRunnable = cc["@mozilla.org/xpcomproxy;1"].getService(ci.nsIProxyObjectManager
       ).getProxyForObject(
          cc["@mozilla.org/event-queue-service;1"].getService(ci.nsIEventQueueService
              ).getSpecialEventQueue(ci.nsIEventQueueService.UI_THREAD_EVENT_QUEUE),
          ci.nsIRunnable, 
          {
            QueryInterface: function(iid) 
            {
              if(iid.equals(ci.nsISupport) || iid.equals(ci.nsIRunnable)) 
              {
               return this;
              }
              throw Components.results.NS_ERROR_NO_INTERFACE;
            },
            run: function() 
            { 
              if(asyncError.value) 
              {
                dump("AsyncRunner mainJob error: "+asyncError.value+"\n");
              }
              try 
              {
                postJob();
              } catch(ex) {
                dump("AsyncRunner postJob error: "+ex+"\n");
              }
            }
          }
        , 5); // 5 == PROXY_ALWAYS | PROXY_SYNC
    }   
    const asyncRunnable=
    {
      run: function() 
      { 
        try {
          asyncJob();
        } catch(ex) {
          asyncError.value=ex;
        }
        if(postRunnable) postRunnable.run();
      }
    };
    
    this.starter.start(asyncRunnable);
  }  
}

