//*************************************************************************
// HTML Validator
//
//  File: tidyBrowser.js
//  Description: common javascript functions used by the extension
//  Author : Marc Gueury
//  Licence : see licence.txt
//*************************************************************************

//-------------------------------------------------------------

var oTidyUtil;

//-------------------------------------------------------------

function onLoadTidyUtil()
{
  oTidyUtil = new TidyUtil();
}

function onUnloadTidyUtil()
{
  delete oTidyUtil;
  oTidyUtil = null;
}

function tidyUtilOpenUrl(aEvent, aTWC)
{
  var url = aEvent.currentTarget.getAttribute("url");
  //get a navigator window
  var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
  var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
  var win = windowManagerInterface.getMostRecentWindow("navigator:browser");
  if (!win) {
    win = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", "about:blank", null, null);
  }
  var browser = win.document.getElementById("content");    
  browser.loadURI(url);  
}

//-------------------------------------------------------------
// TidyUtil
//-------------------------------------------------------------

function TidyUtil()
{
  netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
  
  // init all the control with the values from the preferences
  var pref_service = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
  this.branch = pref_service.getBranch("tidy.options.");
  
  // Check if the preferences exists
  this.setDefaultValueBool( "show-warnings",        true );
  this.setDefaultValueInt ( "show-errors",          6 );
  this.setDefaultValueInt ( "accessibility-check",  -1 );
  this.setDefaultValueBool( "viewsource_disable",   false );

  this.setDefaultValueBool( "indent",               false );
  this.setDefaultValueInt ( "indent-spaces",        2 );
  this.setDefaultValueBool( "uppercase-tags",       false );
  this.setDefaultValueBool( "uppercase-attributes", false );
  this.setDefaultValueInt ( "wrap",                 68 );
  this.setDefaultValueBool( "output-xhtml",         false );
  this.setDefaultValueBool( "output-html",          false );
  this.setDefaultValueBool( "clean",                false );
  this.setDefaultValueChar( "filter",               "" );
  this.setDefaultValueChar( "doctype",              "auto" );
  this.setDefaultValueChar( "output-encoding",      "utf8" );
  
  this.setDefaultValueBool( "debug",                false );
  this.setDefaultValueChar( "diaglog_dir",          "" );
  this.setDefaultValueChar( "browser_icon",         "icon_only" );
  this.setDefaultValueBool( "browser_disable",      false );
  this.setDefaultValueBool( "highlight_line",       true );
  this.setDefaultValueChar( "validate_domain_type", "all" );
  this.setDefaultValueChar( "validate_domain_list", "" );

  try
  {
    // Tidy XPCOM extension
    var tidy_inst = Components.classes["@mozilla.org/tidy;1"].createInstance();
    this.tidy = tidy_inst.QueryInterface(Components.interfaces.nsITidy);
  } 
  catch(e) 
  {
    // Do nothing, later code will handle the error
    dump("@mozilla.org/tidy;1 library not found or incompatible\n");
  }
  
  if( this.getBoolPref("debug") && this.getCharPref("diaglog_dir")!="" )
  {
    this.tidy.enableDebug( this.getCharPref("diaglog_dir") );    
  }
  
  this.buildFilterArray();
  
  try 
  {
    var pref_branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    this.defaultLanguage = pref_branch.getCharPref("general.useragent.locale");
  }
  catch (ex) {}  

}

TidyUtil.prototype =
{  
  branch       : null,
  tidy         : null,  
  filterArray  : null,
  stringBundle : null,
  defaultLanguage : "en-US",

  getPrefConfig : function()
  {
    var sConfig = "";
   
    sConfig += this.prefConfigLineBool( "show-warnings" );
    sConfig += this.prefConfigLineInt ( "show-errors" );
    sConfig += this.prefConfigLineBool( "indent" );
    sConfig += this.prefConfigLineInt ( "indent-spaces" );
    sConfig += this.prefConfigLineBool( "uppercase-tags" );
    sConfig += this.prefConfigLineBool( "uppercase-attributes" );
    sConfig += this.prefConfigLineInt ( "wrap" );
    sConfig += this.prefConfigLineBool( "output-xhtml" );
    sConfig += this.prefConfigLineBool( "output-html" );
    sConfig += this.prefConfigLineBool( "clean" );
    sConfig += this.prefConfigLineChar( "doctype" );
    sConfig += this.prefConfigLineChar( "output-encoding" );

    // accessibility check "-1" is a trick for removing
    // the table summary and img alt warnings.
    // 0 is the default value
    if( this.getIntPref( "accessibility-check" )>0 )
    {
      sConfig += this.prefConfigLineInt ( "accessibility-check" );
    }
    
    return sConfig;
  },
  
  //-------------------------------------------------------------------------

  getBoolPref : function( pref )
  {
    return( this.branch.getBoolPref( pref ) );
  },
  setBoolPref : function(name, value)
  {
    this.branch.setBoolPref( name, value );
  },
  getIntPref : function( pref )
  {
    return( this.branch.getIntPref( pref ) );
  },
  setIntPref : function(name, value)
  {
    this.branch.setIntPref( name, value );
  },
  getCharPref : function( pref )
  {
    return( this.branch.getCharPref( pref ) );
  },
  setCharPref : function(name, value)
  {
    this.branch.setCharPref( name, value );
  },
  
  //-------------------------------------------------------------------------

  prefConfigLineBool : function( pref )
  {
    return( pref + " " + (this.branch.getBoolPref( pref )?"yes":"no") + "\n" );
  },
  prefConfigLineInt : function( pref )
  {
    return( pref + " " + this.branch.getIntPref( pref ) + "\n" );
  },
  prefConfigLineChar : function( pref )
  {
    return( pref + " " + this.branch.getCharPref( pref ) + "\n" );
  },

  //-------------------------------------------------------------------------

  initCheckbox : function( name )
  {
    var control_name = "tidy.options."+name
    document.getElementById(control_name).checked = this.branch.getBoolPref( name );
  },
  saveCheckbox : function( name )
  {
    var control_name = "tidy.options."+name
    this.branch.setBoolPref( name, document.getElementById(control_name).checked );
  }, 
  initTextbox : function( name )
  {
    var control_name = "tidy.options."+name
    document.getElementById(control_name).value = this.branch.getIntPref( name );
  },
  saveTextbox : function( name )
  {
    var control_name= "tidy.options."+name
    this.branch.setIntPref( name, document.getElementById(control_name).value );
  },
  initCharTextbox : function( name )
  {
    var control_name = "tidy.options."+name
    document.getElementById(control_name).value = this.branch.getCharPref( name );
  },
  saveCharTextbox : function( name )
  {
    var control_name= "tidy.options."+name
    this.branch.setCharPref( name, document.getElementById(control_name).value );
  },
  initComboBox : function( name ) 
  {
    var control_name = "tidy.options."+name
    document.getElementById(control_name).value = this.branch.getCharPref( name );
  },
  saveComboBox : function( name )
  {
    var control_name= "tidy.options."+name
    this.branch.setCharPref( name, document.getElementById(control_name).value );
  },

  //-------------------------------------------------------------------------
  
  setDefaultValueBool : function(name, value)
  {
    if( !this.branch.prefHasUserValue(name) ) 
    {
      this.branch.setBoolPref( name, value );
    }
  },
  setDefaultValueInt : function(name, value)
  {
    if( !this.branch.prefHasUserValue(name) ) 
    {
      this.branch.setIntPref( name, value );
    }
  },  
  setDefaultValueChar : function(name, value)
  {
    if( !this.branch.prefHasUserValue(name) ) 
    {
      this.branch.setCharPref( name, value );
    }
  },  

  //-------------------------------------------------------------------------

  /**
   * Build the Filter array
   */
  buildFilterArray : function()
  {
    // List of the shown errors and warnings
    this.filterArray = new Array();
    /// Reset the fast filter in the tidylib
    this.tidy.resetFilter();
        
    // List of the hidden errors and warnings
    var filterString = this.getCharPref( "filter" );
    if( filterString.length>0 )
    {
      var filterHideArray = filterString.split(',');
      for( var o in filterHideArray ) 
      {
        var errorId = filterHideArray[o];  
        this.filterArray[errorId] = false;
        this.tidy.filterMsg( filterHideArray[o] );
      }  
    }
  },
  
  /**
   * Save the filter array in pref
   */
  saveFilterArrayInPref : function()
  {  
    var value = "";
    var bFirst = true;
    
    // Filter List
    for( var o in this.filterArray )
    {
      if( this.filterArray[o]==false )
      {
        if( bFirst )
        {
          bFirst = false;
        }
        else
        {
          value =value + ",";
        }
        value = value + o;
      }
    }
    this.setCharPref( "filter", value );
  },
  
  /** __ saveFile ________________________________________________________________
   * 
   * Utility function to save data to a file
   */   
  /// XXXXXXXXX NOT USED ANYMORE 
  saveFile : function(data, url)
  {
    const MODE =  0x2A; // MODE_WRONLY | MODE_CREATE | MODE_TRUNCAT
    const PERM = 00644; // PERM_IRUSR | PERM_IWUSR | PERM_IRGRP | PERM_IROTH
  
    try 
    {
      var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
      file.initWithPath(url);
      var os = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
      os.init(file, MODE, PERM, 0);
      os.write(data, data.length);
    } 
    catch (ex) 
    {
      alert(ex);
    }
  },
  
  /** __ selectionOn ___________________________________________________________
   * 
   * Reenable the selection after a new page is shown.
   * Why is it needed ? Mystery ?
   */   
  selectionOn : function(xulBrowser)
  {
    const nsISelectionController = Components.interfaces.nsISelectionController;  
    var selCon = xulBrowser.docShell
       .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
       .getInterface(Components.interfaces.nsISelectionDisplay)
       .QueryInterface(nsISelectionController);
    selCon.setDisplaySelection(nsISelectionController.SELECTION_ON);
  },
  
  /** __ cleanupDialog ___________________________________________________________
   * 
   * Call the cleanup dialog box
   */   
  cleanupDialog : function(aNbError, aHtml, aWinArg)
  {
    if( aNbError>0 )
    {
      // Show a confirmation dialog 
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                          .getService(Components.interfaces.nsIPromptService);
      var flags=promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0 +
                promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1;
      var stringBundle = document.getElementById("tidy.string.bundle");
      var err = aNbError+" "+stringBundle.getString( (aNbError>1?"tidy_errors":"tidy_error") ); 
      var msg = stringBundle.getFormattedString("tidy_cleanup_error", [err]);

      var result = promptService.confirmEx(window, this.getString("tidy_cleanup"), msg, flags, null, null, null, null, {} );
            
      if( result==1 )
      {   
        return;
      }
    }
    openDialog(
               "chrome://tidy/content/tidyCleanup.xul",
               "",
               "centerscreen,dialog=no,chrome,resizable",
               aHtml,
               aWinArg
             );
   },
   
  /** __ getString ___________________________________________________________
   */ 
  getString : function(aName)
  {
    if( this.stringBundle==null ) 
    {
      this.stringBundle = document.getElementById("tidy.string.bundle");
      if( this.stringBundle==null ) 
      {
        return "";
      }
    }
    return this.stringBundle.getString( aName );
  },
  
  /** __ isInDomainList ___________________________________________________________
   */ 
  isInDomainList : function(doc)
  {
    try 
    {
      if( doc.location.protocol=="about:" )
      { 
        return false;
      }

      var type = this.getCharPref("validate_domain_type");
      if( type=="all" || doc.location.protocol=="file:" )
      {
        return true;
      }
      
      var host = doc.location.host;
      var domainArray = this.getCharPref("validate_domain_list").split(',');
      
      var o;
      if( type=="only" )
      {
        for (o in domainArray) 
        {
          if( host.search( domainArray[o])!=-1 )
          {
            return true;
          }
        }
        return false;
      }
      else if( type=="except" )
      {
        for (o in domainArray) 
        {
          if( host.search( domainArray[o])!=-1 )
          {
            return false;
          }
        }
        return true;
      }
    }
    catch(ex)
    {}
    return true;
  },

  /** __ translation ___________________________________________________________
   *
   * Load the translation from the tidy.properties file in the tidy lib
   */ 
  translation : function()
  {
    if( this.tidy.initTranslation() )
    {
      try
      {
        // Does not load the us translation when not in debug mode
        if( oTidyUtil.getBoolPref("debug") || this.defaultLanguage!="en-US" )
        {
          var id_list ={value:""};
          this.tidy.getIdOfAllErrors( id_list );
          var id_array = id_list.value.split(',');
          var t = "";
          for (var id in id_array) 
          {
            t += this.getString("tidy_"+id_array[id]) + "\n";
          }
          this.tidy.addTranslations( t );
          var prefix = this.getString("tidy_prefix") ;
          var linecol = this.getString("tidy_linecol")+" ";

          this.tidy.setTranslationPrefix( prefix, linecol );
          this.tidy.checkTranslation();
         
          this.buildFilterArray();
        }    
        /* XXXXXXXXXXX Does not work when placed here
        // Translate the description of the extension. 
        var tidydesc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.{3b56bcc7-54e5-44a2-9b44-66c3ef58c13e}.");
        var descstr  = Components.classes[ "@mozilla.org/supports-string;1" ].createInstance( Components.interfaces.nsISupportsString );
        descstr.data = this.getString("tidy_extension_desc");
        tidydesc.setComplexValue("description", Components.interfaces.nsISupportsString, descstr);   
        */
      }
      catch( e )
      {
        oTidyUtil.tidy.log( '<ERROR>translation exception, defaultLanguage= ' + this.defaultLanguage );
      }
    }
  }
}

//-------------------------------------------------------------
// TidyResult
//-------------------------------------------------------------

function TidyResult( doc )
{
  if( doc!=null )
  {
    this.lastURL = doc.URL;
  }
}

TidyResult.prototype =
{  
  lastURL : null,
  iNbError : 0,
  iNbWarning : 0,
  iNbAccessWarning : 0,
  iNbHidden : 0,
  bInDomainList : true,
  bValidated : false,
  bUConvFailed : false,
  bEmpty : false,
  
  /** __ getPluralString ______________________________________________________
   */    
  getPluralString : function(nb,s)
  {
    if( nb==0 ) 
    {
      return "tidy_0_"+s+"s";
    }
    else if( nb==1 ) 
    {
      return "tidy_"+s;
    }
    else
    {
      return "tidy_"+s+"s";
    }
  },
  
  /** __ getErrorString ______________________________________________________
   * 
   * Build a error string for a page
   */    
  getErrorString : function()
  {
    var str;
    if( this.bInDomainList )
    {
      if( this.bEmpty )
      {
        str = oTidyUtil.getString('tidy_empty'); 
      } 
      else if( this.bUConvFailed )
      {
        str = oTidyUtil.getString('tidy_invalid_char');
      }
      else 
      {
        str =       
          this.iNbError    + " " + oTidyUtil.getString( this.getPluralString( this.iNbError, "error") ) + " / " + 
          this.iNbWarning  + " " + oTidyUtil.getString( this.getPluralString( this.iNbWarning, "warning")  ) +
          ( this.iNbAccessWarning==0 ? "" : " / "+this.iNbAccessWarning+" "+oTidyUtil.getString( this.getPluralString( this.iNbAccessWarning, "access_warning") )) +
          ( this.iNbHidden==0 ? "" : " ("+this.iNbHidden+" "+oTidyUtil.getString("tidy_hidden")+")" );
      }
    }
    else
    {
      str = oTidyUtil.getString("tidy_not_in_domain");
    }
    return str;
  },
   
  /** __ getErrorIcon _______________________________________________________
   * 
   * Build a error icon for a page
   */    
  getErrorIcon : function()
  {
    var icon = "chrome://tidy/skin/good";
    if( this.bInDomainList )
    {
      if( this.bUConvFailed )
      {
        icon = "chrome://tidy/skin/charset";
      }
      else if( this.bEmpty )
      {
        icon = "chrome://tidy/skin/empty";
      }
      else if( this.iNbError>0 )
      {
        icon = "chrome://tidy/skin/error";
      }
      else if( this.iNbWarning>0 )
      {
        icon = "chrome://tidy/skin/warning";
      }        
      else if( this.iNbHidden>0 )
      {
        icon = "chrome://tidy/skin/hidden";
      }  
    }
    else
    {
      icon = "chrome://tidy/skin/disabled";
    }
    return icon;
  }
}