//*************************************************************************
// HTML Validator
//
//  File: tidyBrowser.js
//  Description: javascript to validate the HTML in the view source window
//  Author : Marc Gueury
//  Licence : see licence.txt
//*************************************************************************

//-------------------------------------------------------------
// Public functions
//-------------------------------------------------------------

var oTidyValidate;

function onLoadTidyValidate()
{
  onLoadTidyUtil();
  oTidyValidate = new TidyValidate();
  oTidyValidate.start();
  
  // Hide the validator here to avoid a "splash" effect  
  if( oTidyUtil.getBoolPref( "viewsource_disable" ) )
  {
    oTidyValidate.hideValidator( true );
  }
  
  // Call the original function of the View Source dialog.
  onLoadViewSource();
    
  // Register the onLoad trigger, when the page is loaded -> validate
  getBrowser().addEventListener("load", tidyValidateHtml, true);

}

function onUnloadTidyValidate() 
{
  onUnloadTidyUtil();
  oTidyValidate.stop();
  delete oTidyValidate;
  oTidyValidate = null;
}

function tidyValidateABC()
{
  oTidyValidate.validateHtml( "<html><abc></html>" );
}

function tidyValidateHtml()
{
  var box = document.getElementById("tidy-validate-box");
  var value = !box.hidden;
  if( 
   (   window._content.document.contentType == "text/html"
    || window._content.document.contentType == "application/xhtml+xml"
   ) 
   && !box.hidden
  )
  {
    oTidyValidate.validateHtmlFromNode();
  }
  else
  {
    oTidyValidate.hideValidator( true );
  }
  oTidyValidate.stopProgressMeter();
}

function tidyOptions()
{
  openDialog(
             "chrome://tidy/content/tidyOptions.xul",
             "",
             "centerscreen,dialog=no,chrome,resizable,dependent,modal"
            );
  // rebuild the filter array            
  oTidyUtil.buildFilterArray();            
  // remove the color from the lines
  oTidyValidate.removeColorFromLines();
  // revalidate with the new options            
  oTidyValidate.validateHtmlFromNode();      
}

function tidyCleanup()
{
  var sHtml = oTidyValidate.getHtmlFromNode(); 
  oTidyUtil.cleanupDialog( oTidyValidate.iNbError, sHtml, window.arguments );
}

function tidyHideValidator()
{
  var box = document.getElementById("tidy-validate-box");
  var value = !box.hidden;
  oTidyValidate.hideValidator( value );
}

function tidyValidateLoadExplainError()
{
  oTidyUtil.selectionOn( document.getElementById("tidy-explain-error") );
}

//-------------------------------------------------------------
// TidyValidate
//-------------------------------------------------------------

function TidyValidate()
{
  this.data = new Array(); //Data for each row
  this.type = new Array(); //Type of data 
  this.atoms= new Array(); //Atoms for tree's styles  
}

TidyValidate.prototype =
{
  // xul elements
  xulTree: null,
  xulMenuHide: null,
  xulScrollBar: null,
  xulExplainError: null,
  xulButtonCleanup: null,

  // Tidy initialized or not yet
  bIsValidated: false,
  iLastErrorId: -1,
  iNbError: 0,
  
  // Style
  STYLE_NORMAL  : 1,
  STYLE_RED     : 2,
  STYLE_SUMMARY : 3,

  // Tree interface
  rows: 0,          // number of rows of the tree
  tree: null,       // tree interface
  
  // Horizontal scrolling
  hScrollPos: 0,
  hScrollMax: 0,
  datapresent: "datapresent",

  /** __ treeView ________________________________________________________________
   * 
   * Implementation of all needed function for the tree interface
   */    
  set rowCount(c) { throw "rowCount is a readonly property"; },
  get rowCount() { return this.rows; },
  getCellText: function(row, column) 
  {
    return this.data[row].substr(this.hScrollPos).match(/^.*/);
  },
  setCellText: function(row, column, text) {},
  setTree: function(tree) { this.tree = tree; },
  isContainer: function(index) { return false; },
  isSeparator: function(index) { return false; }, 
  isSorted: function() {},
  getLevel: function(index) { return 0; },
  getImageSrc: function(row, column) {},
  getCellProperties: function(row, col, props) 
  {
    props.AppendElement(this.atoms[this.type[row]]);
  },
  getColumnProperties: function(column, elem, prop) {}, 
  getRowProperties: function(row, props) { },

  isContainerOpen: function(index) { },
  isContainerEmpty: function(index) { return false; },
  canDropOn: function(index) { return false; },
  canDropBeforeAfter: function(index, before) { return false; },
  drop: function(row, orientation) { return false; },
  getParentIndex: function(index) { return 0; },
  hasNextSibling: function(index, after) { return false; },
  getProgressMode: function(row, column) { },
  getCellValue: function(row, column) { },
  toggleOpenState: function(index) { },
  cycleHeader: function(col, elem) { },
  selectionChanged: function() { },
  cycleCell: function(row, column) { },
  isEditable: function(row, column) { return false; },
  performAction: function(action) { },
  performActionOnRow: function(action, row) { },
  performActionOnCell: function(action, row, column) { },

  /** __ addRow ________________________________________________________________
   * 
   * Tree utility function : add a row
   */    
  addRow: function(row, type) 
  { 
    // Add the row
    this.type.push(type);
    this.rows = this.data.push(row);
    if (row.length > this.hScrollMax)
    {
      this.sethScroll(row.length);
    }
  }, 
  
  /** __ rowCountChanged ______________________________________________________
   * 
   * Tree utility function : notice the tree that the row count is changed
   */    
  rowCountChanged: function(index, count)
  {
    if( this.tree )
    {
      var lvr = this.tree.getLastVisibleRow();
      this.tree.rowCountChanged(index, count);
      // If the last line of the tree is visible on screen, we will autoscroll
      if (lvr >= index) this.tree.ensureRowIsVisible(this.rows-1);
    }
  },

  /** __ sethScroll ___________________________________________________________
   * 
   * Horizontal scrolling function
   */
  sethScroll: function(max) 
  {
    // Set the new maximum value and page increment to be 5 steps
    var maxpos = this.xulScrollBar.attributes.getNamedItem("maxpos");
    var pageincrement=this.xulScrollBar.attributes.getNamedItem("pageincrement");
    maxpos.value = (max>2 ? max-3 : 0);
    pageincrement.value = max/5;
    this.hScrollMax = max;
  },
  
  /** __ hScrollHandler _____________________________________________________
   * 
   * Horizontal scrolling function : call back function
   */
  hScrollHandler: function() 
  {
    var base = oTidyValidate;
    var curpos = base.xulScrollBar.attributes.getNamedItem("curpos").value;
    if (curpos != base.hScrollPos) 
    {
      base.hScrollPos = curpos;
      base.tree.invalidate();
    }
  },
  
  /** __ start ________________________________________________________________
   * 
   * Initialisation and termination functions
   */    
  start : function()
  {
    // Tree
    this.xulTree = document.getElementById("tidy-validate-tree");
    this.xulTree.treeBoxObject.view = this;

    // Set scrollbar
    this.xulScrollBar = document.getElementById("tidy-validate-tree-scroll");
    setInterval(this.hScrollHandler,100);

    // Set explain error HTML
    this.xulExplainError = document.getElementById("tidy-explain-error");
    this.xulExplainError.addEventListener("load", tidyValidateLoadExplainError, true);

    // Enable the selection in the explain-error (Firefox 1.0 only)
    // Firefox 1.4 raise an exception but it works out of the box
    try
    {
      var selCon = this.xulExplainError.docShell
         .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
         .getInterface(Components.interfaces.nsISelectionDisplay)
         .QueryInterface(nsISelectionController);
      selCon.setDisplaySelection(nsISelectionController.SELECTION_ON);
      selCon.setCaretEnabled(true);
      selCon.setCaretVisibilityDuringSelection(true);
    }
    catch(ex)
    {
    }

    // Tree Style (called Atoms)
    var aserv=Components.classes["@mozilla.org/atom-service;1"].
               createInstance(Components.interfaces.nsIAtomService);
    this.atoms[this.STYLE_NORMAL]  = aserv.getAtom("StyleNormal");
    this.atoms[this.STYLE_RED]     = aserv.getAtom("StyleRed");
    this.atoms[this.STYLE_SUMMARY] = aserv.getAtom("StyleSummary");
    
    // Menu 
    this.xulMenuHide = document.getElementById('tidy.hide');
    this.xulMenuHide.setAttribute("checked", "false");
    
    // Cleanup button
    this.xulButtonCleanup = document.getElementById("tidy-cleanup-button");
    
    // Load start page
    this.loadHelp("tidy_start.html");
  },
  
  stop : function()
  {
    this.xulTree.treeBoxObject.view = null;
    this.xulTree = null;
    this.xulScrollBar = null;
  },  
  
  /** __ clear ________________________________________________________________
   * 
   * Clear the tree
   */    
  clear : function()
  {
    var oldrows = this.rows;
    if( oldrows>0 )
    {
      this.rows = 0;
      this.data = new Array();
      this.type = new Array();
      this.rowCountChanged(0,-oldrows);
      this.hScrollMax = 0;
      this.sethScroll(0);
    }
  },  
    
  /** __ stopProgressMeter ____________________________________________________
   * 
   * Stop the progress meter
   */    
  stopProgressMeter : function()
  {
    var meter = document.getElementById("tidy-progress");
    meter.setAttribute("value", 1);
    meter.setAttribute("value", 0);
    meter.setAttribute("mode", "determined");
    meter.setAttribute("hidden", "true");
  },
  
  /** __ validateHtml ________________________________________________________
   * 
   * Validate the HTML and add the results in the tree
   */    
  validateHtml : function( aHtml )
  {
    // Set the initialization flag
    this.bIsValidated = true;
    
    // Load the translation (if not done)
    oTidyUtil.translation();
    
    // Show in the menu that the validator is enabled
    this.xulMenuHide.setAttribute("checked", "true");
    
    // The inout arguments need to be JavaScript objects
    var nbError = {value:0};
    var nbWarning = {value:0};
    var nbAccessWarning = {value:0};
    var nbHidden = {value:0};
    var error ={value:"---"};
    var accessLevel = oTidyUtil.getIntPref( "accessibility-check" );
    
    oTidyUtil.tidy.getErrorsInHTML( aHtml, oTidyUtil.getPrefConfig(), accessLevel, error, nbError, nbWarning, nbAccessWarning, nbHidden );
  
    this.clear();
    
    // Show an error if the mime/type is not text/html or xhtml
    if( window._content.document.contentType != "text/html" 
     && window._content.document.contentType != "application/xhtml+xml" )
    {
      this.addRow( oTidyUtil.getString("tidy_not_html")+" "+window._content.document.contentType+"\r\n", this.STYLE_RED);
    }
    
    var oldrows = this.rowCount;
    var multi = error.value.split('\n');
    var colorLines = new Array();
    var hiddenNb = 0;
    for (var o in multi) 
    {
      var d = multi[o];
      if( d.search( "\r" ) >= 0 )
      {
        d = d.substring(0, d.search( "\r" ) );
      }
      var aLine = {value:-1};
      var aColumn = {value:-1};
      var aType = {value:-1};
      var aText = {value:""};
      var aErrorId = {value:-1};
      var aArg1 = {value:""};
      var aArg2 = {value:""};
      var aArg3 = {value:""};

      oTidyUtil.tidy.analyzeLine( d, aLine, aColumn, aType, aText, aErrorId, aArg1, aArg2, aArg3 );          

      var line = aLine.value;
      var col = aColumn.value;
      var errorId = aErrorId.value;
      var type = aType.value;
      
      if( line>0 && col>0 )
      {
        colorLines[line] = true;
      }      

      if( type==4 ) // error
      {
        this.addRow( d + "\r\n", this.STYLE_RED);
      }
      else
      {
        this.addRow( d + "\r\n", this.STYLE_NORMAL);
      }       
    }
    if( nbError.value>=oTidyUtil.getIntPref("show-errors") )
    {
      this.addRow( oTidyUtil.getString("tidy_too_many_error")+"\r", this.STYLE_SUMMARY );
    }
    
    var res = new TidyResult();
    res.iNbError = nbError.value ;
    res.iNbWarning = nbWarning.value;
    res.iNbAccessWarning = nbAccessWarning.value;
    res.iNbHidden = nbHidden.value;   
    var last_line = this.addRow( res.getErrorString()+"\r\n", this.STYLE_SUMMARY );
    
    // save the value for the cleanup button
    this.iNbError = nbError.value;

    this.rowCountChanged(oldrows,(this.rows-oldrows)); 
    
    if( oTidyUtil.getBoolPref( "highlight_line" ) )
    {
      this.colorizeLines( colorLines );
    }
  },  
  
  /** __ onSelect ____________________________________________________
   * 
   * Select function
   */   
  onSelect: function()
  {
    var selection = this.xulTree.view.selection;

    // A warning/error line is something like : 'line 10 column 20 ...'
    var d = this.data[selection.currentIndex];
    
    // No line is selected ?
    if( !d ) return;
    
    // Remove the '\r\r\n' (NT) at the end of the string
    //            '\r\n' (Linux)
    d = d.substring(0, d.search( "\r" ) );

    // Call tidy to get the ID and arguments of the error.    
    var aLine = {value:-1};
    var aColumn = {value:-1};
    var aType = {value:-1};
    var aText = {value:""};
    var aErrorId = {value:-1};
    var aArg1 = {value:""};
    var aArg2 = {value:""};
    var aArg3 = {value:""};

    oTidyUtil.tidy.analyzeLine( d, aLine, aColumn, aType, aText, aErrorId, aArg1, aArg2, aArg3 );          

    var src = "tidy_start.html"

    var line = aLine.value;
    var col = aColumn.value;
    var type = aType.value;
    
    if( line>0 && col>0 )
    {
      var arg1=aArg1.value;
      var arg2=aArg2.value;
      var arg3=aArg3.value;

      // build the array of potentially selectable strings
      var aToSelect = new Array();
      if( arg1.length>0 )
      {
        aToSelect.push( arg1 );
        if( arg2.length>0 )
        {
          aToSelect.push( arg2 );
          if( arg3.length>0 )
          {
            aToSelect.push( arg3 );
          }
        }
      }
      this.goToLineCol( line, col, aToSelect );
       
      var error_id = aErrorId.value;
      this.iLastErrorId = error_id;
      
      if( type==3 )
      {
        var pos1 = d.indexOf( '[' );
        var pos2 = d.indexOf( ']' );
        var access_id = d.substring( pos1+1, pos2 );    
        // alert( access_id );
        src = "access_"+access_id+".html";
      }
      else if( 
          error_id==1
       || error_id==3
       || error_id==4
       || error_id==6
       || error_id==7
       || error_id==8
       || error_id==9
       || error_id==11
       || error_id==12
       || error_id==15
       || error_id==21
       || error_id==22
       || error_id==23
       || error_id==25
       || error_id==29
       || error_id==32
       || error_id==41
       || error_id==44
       || error_id==49
       || error_id==50
       || error_id==51
       || error_id==53
       || error_id==54
       || error_id==55
       || error_id==58
       || error_id==60
       || error_id==64
       || error_id==65
       || error_id==66
       || error_id==70
       || error_id==73
       || error_id==74
       || error_id==83
       || error_id==86
      )
      {
        src = "tidy_"+error_id+".html";
      }
      else if( error_id>0 )
      {
        src = "tidy_blank.html";
      }
    }

    this.loadHelp(src);
  },

  /** __ loadHelp ____________________________________________________
   * 
   * load an help file
   */   
  loadHelp : function(src)
  {
    var url = "chrome://tidy/content/help/"+oTidyUtil.defaultLanguage+"/"+src;
    try
    {
      var req = new XMLHttpRequest();
      req.open('GET', url, false); 
      req.send(null);
    }
    catch(ex)
    {
      // help file is not translated - default to the en-US
      url = "chrome://tidy/content/help/en-US/"+src;
    }
    this.xulExplainError.loadURI( url );   
  },
     
  /** __ onTreeHide ____________________________________________________
   * 
   * Hide a message when right clicking on it
   */   
  onTreeHide : function()
  {
    if( this.iLastErrorId>0 )
    {
      // The inout arguments need to be JavaScript objects
      var aErrorDesc = {value:""};
      oTidyUtil.tidy.getErrorDescription( this.iLastErrorId, aErrorDesc );
    
      // Show a confirmation dialog 
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                          .getService(Components.interfaces.nsIPromptService);
      var flags=promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0 +
                promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1;
      var result = promptService.confirmEx(window,oTidyUtil.getString("tidy_filter_title"),
                   oTidyUtil.getString("tidy_filter_msg") + aErrorDesc.value,
                   flags, null, null, null, null, {} );
     
      if( result==0 )
      {   
        oTidyUtil.filterArray[this.iLastErrorId] = false;
        oTidyUtil.saveFilterArrayInPref();
      
        // revalidate with the new options            
        // oTidyValidate.validateHtmlFromNode();   
        tidyOptions();
      }
    }
  },
  
  /** __ onTreeCopy ____________________________________________________
   * 
   * Copy the current error message in clipboard
   */   
  onTreeCopy : function()
  {
    var selection = this.xulTree.view.selection;
    var data = "", rStart = {}, rEnd = {};
    var ranges = selection.getRangeCount();
    for (var range=0; range<ranges; range++) 
    {
      selection.getRangeAt(range, rStart, rEnd);
      if (rStart.value >=0 && rEnd.value >=0) 
      {
        for (var row=rStart.value; row<=rEnd.value; row++) 
        {
          data += this.data[row];
	}
      }
    }
    if( oTidyUtil.getBoolPref( "debug" ) )
    {
      // In debug mode, show the ID and the desc of the error
      var aErrorDesc = {value:""};
      oTidyUtil.tidy.getErrorDescription( this.iLastErrorId, aErrorDesc );

      data += "ErrorId: "+this.iLastErrorId + " / Desc: " + aErrorDesc.value;
    }

    if (data) 
    {
      // clipboard helper
      try
      {
        const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	clipboardHelper.copyString(data);
      } 
      catch(e) 
      {
        // do nothing, later code will handle the error
	dump("Unable to get the clipboard helper\n");
      }
    }    
  },

  /** __ onTreeSelectAll ____________________________________________________
   * 
   * Select all
   */   
  onTreeSelectAll : function()
  {
    return this.xulTree.view.selection.selectAll();
  },
  
  /** __ validateHtmlFromNode ___________________________________________
   *
   * Get the HTML from the view source tree then validate it.
   */   
  validateHtmlFromNode: function()
  {
    this.validateHtml( this.getHtmlFromNode() );
  },
  

  /** __ getHtmlFromNode _________________________________________________
   *
   * Build the HTML from the text of all the source view nodes
   * This is done to avoid another network request
   */   
  getHtmlFromNode: function()
  {
    var viewsource = window._content.document.body;
    var sHtml = "";
    var pre;
    
    //
    // Walk through each of the text nodes
    //
    for (var i = 0; i < viewsource.childNodes.length; i++ ) 
    {
      pre = viewsource.childNodes[i];
  
      var treewalker = window._content.document.createTreeWalker(pre, NodeFilter.SHOW_TEXT, null, false);
  
      for (var textNode = treewalker.firstChild();
           textNode;
           textNode = treewalker.nextNode()) 
      {  
        sHtml = sHtml + textNode.data;        
      }
    }
   
    return sHtml;
  },  
  
  
  /** __ colorizeLines ______________________________________________________
   * 
   * Color a array of lines
   */
  colorizeLines: function( colorLines )
  {
    for (var line in colorLines) 
    {
      l = parseInt( line );  
      this.colorizeOneLine( l );
    }
  },

  /** __ colorizeOneLine ____________________________________________________
   * 
   * Color one line of the HTML source
   *
   * @param line : (number) line number
   */
  colorizeOneLine: function( line )
  {
    // This code is inspired by the goToLine of viewSource.js 
    // it is the same, except that the range is highlighted
    var viewsource = window._content.document.body;
    var pre;
    for (var lbound = 0, ubound = viewsource.childNodes.length; ; ) {
      var middle = (lbound + ubound) >> 1;
      pre = viewsource.childNodes[middle];
  
      var firstLine = parseInt(pre.id.substring(4));
  
      if (lbound == ubound - 1) {
        break;
      }
  
      if (line >= firstLine) {
        lbound = middle;
      } else {
        ubound = middle;
      }
    }
  
    var result = {};
    var found = findLocation(pre, line, null, -1, false, result);
  
    if (!found) {
      return;
    }
    
    this.colorizeRange(result.range);
  },  

  /** __ colorizeRange ____________________________________________________
   * 
   * Color a range of the HTML source
   *
   * @param range : (range) the range 
   */
  colorizeRange: function( range )  
  {
    var doc = window._content.document;
    var node = doc.createElement("span");
    node.setAttribute("style", "background-color: #DDDDFF;");
    node.setAttribute("id", "__firefox-tidy-id");
  
    // This code is inspired by the highlight function of browser.js 
    var startContainer = range.startContainer;
    var startOffset = range.startOffset;
    var endOffset = range.endOffset;
    var docfrag = range.extractContents();
    var before = startContainer.splitText(startOffset);
    var parent = before.parentNode;
    node.appendChild(docfrag);
    parent.insertBefore(node, before);
    return node;
  },
  
  /** __ removeColorFromLines ___________________________________________
   * 
   * Remove the color added to the lines
   */
  removeColorFromLines: function()  
  {
    var doc = window._content.document;
    var elem = null;

    while ((elem = doc.getElementById("__firefox-tidy-id"))) 
    {
      var child = null;
      var docfrag = doc.createDocumentFragment();
      var next = elem.nextSibling;
      var parent = elem.parentNode;
      while((child = elem.firstChild)) 
      {
        docfrag.appendChild(child);
      }
      parent.removeChild(elem);
      parent.insertBefore(docfrag, next);
    }
    return;
  },    
  
  /** __ goToLineCol ____________________________________________________
   * 
   * Go to the line and col in the HTML source
   *
   * @param line : (number) line
   * @param col  : (number) column
   */
  goToLineCol: function(line, col, aToSelect)
  {
    var viewsource = window._content.document.body;
  
    var pre;
    for (var lbound = 0, ubound = viewsource.childNodes.length; ; ) {
      var middle = (lbound + ubound) >> 1;
      pre = viewsource.childNodes[middle];
  
      var firstLine = parseInt(pre.id.substring(4));
  
      if (lbound == ubound - 1) {
        break;
      }
  
      if (line >= firstLine) {
        lbound = middle;
      } else {
        ubound = middle;
      }
    }
  
    var result = {};
    var found = this.getLineColRange(pre, line, col, aToSelect, result);
  
    if (!found) {
      return false;
    }
  
    var selection = window._content.getSelection();
    selection.removeAllRanges();
    selection.QueryInterface(nsISelectionPrivate)
      .interlinePosition = true;	
    selection.addRange(result.range);
    if (!selection.isCollapsed) 
    {
      selection.collapseToEnd();
  
      var offset = result.range.startOffset;
      var node = result.range.startContainer;
      if (offset < node.data.length) {
        // The same text node spans across the "\n", just focus where we were.
        selection.extend(node, offset);
      }
      else {
        // There is another tag just after the "\n", hook there. We need
        // to focus a safe point because there are edgy cases such as
        // <span>...\n</span><span>...</span> vs.
        // <span>...\n<span>...</span></span><span>...</span>
        node = node.nextSibling ? node.nextSibling : node.parentNode.nextSibling;
        selection.extend(node, 0);
      }
    }
  
    var selCon = this.getSelectionController();
    selCon.setDisplaySelection(nsISelectionController.SELECTION_ON);
    selCon.setCaretEnabled(true);
    selCon.setCaretVisibilityDuringSelection(true);
  
    // Scroll the beginning of the line into view.
    selCon.scrollSelectionIntoView(
      nsISelectionController.SELECTION_NORMAL,
      nsISelectionController.SELECTION_FOCUS_REGION,
      true);
  
    gLastLineFound = line;
  
    // needed to avoid problem in Firefox
    var statusbar = document.getElementById("statusbar-line-col");
    if( statusbar )
    {
       statusbar.label = getViewSourceBundle().getFormattedString("statusBarLineCol", [line, 1]);
    }
  
    return true;
  },
  
  /** __ getSelectionController _________________________________________
   *
   * This is to avoid repeatless bugs and change about this function 
   * in Firefox where it becomes tiring
   */
  getSelectionController : function()
  {
     return getBrowser().docShell
     .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
     .getInterface(Components.interfaces.nsISelectionDisplay)
     .QueryInterface(nsISelectionController);
   
  },
  
  /** __ getLineColRange ____________________________________________________
   * 
   * get the range for a line and a column 
   *
   * @param
   */
  getLineColRange : function (pre, line, col, aToSelect, result)
  {
    var curLine = parseInt(pre.id.substring(4));
  
    //
    // Walk through each of the text nodes and count newlines.
    //
    var treewalker = window._content.document
        .createTreeWalker(pre, NodeFilter.SHOW_TEXT, null, false);
  
    //
    // The column number of the first character in the current text node.
    //
    var firstCol = 1;
  
    var found = false;
    for (var textNode = treewalker.firstChild();
         textNode && !found;
         textNode = treewalker.nextNode()) 
    {
  
      //
      // \r is not a valid character in the DOM, so we only check for \n.
      //
      var lineArray = textNode.data.split(/\n/);
      var lastLineInNode = curLine + lineArray.length - 1;
  
      //
      // Check if we can skip the text node without further inspection.
      //

      var nextFirstCol = firstCol;
      if (lineArray.length > 1) 
      {
        nextFirstCol = 1;
      }
      nextFirstCol += lineArray[lineArray.length - 1].length;

      if( 
          lastLineInNode < line || 
          ( lastLineInNode==line && nextFirstCol<=col )
        ) 
      {
        firstCol = nextFirstCol;
        curLine = lastLineInNode;
        continue;
      }
  
      //
      // curPos is the offset within the current text node of the first
      // character in the current line.
      //
      for (var i = 0, curPos = 0;
           i < lineArray.length;
           curPos += lineArray[i++].length + 1) 
      {
  
        if (i > 0) 
        {
          curLine++;
          firstCol = 1;
        }
  
        if (curLine == line && !("range" in result)) 
        {
          // The default behavior is to select 1 character.
          result.range = document.createRange();
          var pos = curPos+col-firstCol;         
          result.range.setStart(textNode, pos);
          result.range.setEnd(textNode, pos+1);
          found = true;  


          // Check if the text does not contain a selectable string

          // get first the maximum len of the selectable string
          var maxlen = 0;
          for( var j=0; j<aToSelect.length; j++ )
          {
            var len = aToSelect[j].length;
            if( len>maxlen ) maxlen = len;
          }
          
          // get the rest of the text of the node where the text starts
          var textAfter = textNode.data.substr( pos );
          
          // we will potentially look 2 node further
          for ( var n=0; n<3; n++ ) 
          {   
            // Search in the selectable string array
            for( var j=0; j<aToSelect.length; j++ )
            {
              var s = aToSelect[j].toLowerCase();
              var len = s.length;
              var s2 = textAfter.substr( 0, len ).toLowerCase();
              if( s==s2 )
              {
                result.range.setEnd(textNode, pos+len);              
              }
              // If it is a tag, '<TAG>', '<TAG' is also good
              else if( s.charAt(0)=='<' && s.charAt(len-1)=='>' )
              {
                s = s.substr(0,len-1);
                s2 = s2.substr(0,len-1);
                if( s==s2 )
                {
                  result.range.setEnd(textNode, pos+len-1);              
                }                          
              }
            }
            
            // If the text searched is bigger than the biggest string to search,
            // stop the search
            len = textAfter.length
            if( len>=maxlen )
            {
              break;
            }
            pos = -len;
            // Go to the next node 
            textNode = treewalker.nextNode()
            if( textNode )
            {
              // Concatenate the next node text
              textAfter += textNode.data; 
            }
            else
            {
              break;
            }
          }
        } 
      }
    }
  
    return found;
  },
  
  /** __ hideValidator ____________________________________________________
   * 
   * Hide the html validator
   *
   * @param bHide : (boolean) hide or not hide 
   */
  hideValidator : function( bHide )
  {
    var box = document.getElementById("tidy-validate-box");
    var splitter = document.getElementById("tidy-splitter");
    box.hidden = bHide;
    splitter.hidden = bHide;
    this.xulMenuHide.setAttribute("checked", !bHide);
    
    // The validation has already been done ?
    if( !bHide && !this.bIsValidated )
    {
      oTidyValidate.validateHtmlFromNode();
    }
  },
  
  /** __ onReload _____________________________________________
   * 
   * Reload the page by skipiing the cache
   */
  
  onReload : function ()
  {
    const webNavigation = getBrowser().webNavigation;
    webNavigation.reload(webNavigation.LOAD_FLAGS_BYPASS_PROXY | webNavigation.LOAD_FLAGS_BYPASS_CACHE);
  }

}

//---------------------------------------------------------------------------
// Copy from browser.js

function tidyExplainErrorOnClick(event)
{
   var target = event.target;
   var linkNode;

   var local_name = target.localName;

   if (local_name) 
   {
     local_name = local_name.toLowerCase();
   }

   switch (local_name) 
   {
     case "a":
     case "area":
     case "link":
       if (target.hasAttribute("href")) 
         linkNode = target;
       break;
     default:
       linkNode = findParentNode(event.originalTarget, "a");
       // <a> cannot be nested.  So if we find an anchor without an
       // href, there is no useful <a> around the target
       if (linkNode && !linkNode.hasAttribute("href"))
         linkNode = null;
       break;
   }
   if (linkNode && event.button == 0 ) 
   {
     openNewWindowWith(linkNode.href, linkNode, false);
     event.preventDefault();
     event.preventBubble();
     return true;
   }
   return true;
}
